package com.zd.system.mapper;

import com.zd.system.api.domain.SysUser;
import com.zd.system.api.laboratory.domain.TotalByID;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * 用户表 数据层
 *
 * @author zd
 */
public interface SysUserMapper {
    /**
     * 根据条件分页查询用户列表
     *
     * @param sysUser 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> selectUserList(SysUser sysUser);

    /**
     * 根据部门ids查询用户列表
     *
     * @param deptIds
     * @return
     */
    public List<SysUser> selectUserListByDeptIds(Long... deptIds);

    /**
     * 根据条件分页查询未已配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> selectAllocatedList(SysUser user);

    /**
     * 根据条件分页查询未分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> selectUnallocatedList(SysUser user);

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
    public SysUser selectUserByUserName(String userName);

    /**
     * 通过用户名查询用户（个人中心）
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
    public SysUser selectUserCenterByUserName(String userName);

    /**
     * 通过用户名查询用户
     *
     * @param nickName 用户真实姓名
     * @return 用户对象信息
     */
    public List<SysUser> selectUserByNickName(String nickName);

    /**
     * 通过用户卡号查询用户
     *
     * @param cardNum 卡号
     * @return 用户对象信息
     */
    public SysUser selectUserByCardNum(String cardNum);


    /**
     * 通过用户卡号查询用户id,用户姓名
     *
     * @param cardNum 卡号
     * @return 用户对象信息
     */
    public SysUser selectUserInfoByCardNum(String cardNum);

    /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
    public SysUser selectUserById(Long userId);

    /**
     * 新增用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int insertUser(SysUser user);

    /**
     * 修改用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int updateUser(SysUser user);

    /**
     * 修改教职工信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int updateTeacherUser(SysUser user);


    /**
     * 修改学员信息
     *
     * @param user 用户信息
     * @return 结果
     */
    public int updateStudentUser(SysUser user);

    /**
     * 修改用户头像
     *
     * @param userName 用户名
     * @param avatar   头像地址
     * @return 结果
     */
    public int updateUserAvatar(@Param("userName") String userName, @Param("avatar") String avatar);

    /**
     * 重置用户密码
     *
     * @param userName 用户名
     * @param password 密码
     * @return 结果
     */
    public int resetUserPwd(@Param("userName") String userName, @Param("password") String password);

    /**
     * 通过用户ID删除用户
     *
     * @param userId 用户ID
     * @return 结果
     */
    public int deleteUserById(Long userId);

    /**
     * 批量删除用户信息
     *
     * @param userIds 需要删除的用户ID
     * @return 结果
     */
    public int deleteUserByIds(Long[] userIds);


    /**
     * 根据ID查询电话号码
     * @param ids
     * @return
     */
    public List<String> queryPhoneByIds(Long... ids);
    /**
     * 查询人数通过部门ID
     *
     * @param ids
     * @return
     */
    public List<TotalByID> queryCountByDeptIds(Long... ids);

    /**
     * 校验用户名称是否唯一
     *
     * @param userName 用户名称
     * @return 结果
     */
    public int checkUserNameUnique(String userName);

    /**
     * 校验手机号码是否唯一
     *
     * @param phonenumber 手机号码
     * @return 结果
     */
    public SysUser checkPhoneUnique(String phonenumber);

    /**
     * 校验手机号码是否唯一排除自己
     *
     * @param userInfo 手机号码,用户账户
     * @return 结果
     */
    public SysUser checkPhoneUniqueNotOwn(SysUser userInfo);

    /**
     * 校验email是否唯一
     *
     * @param email 用户邮箱
     * @return 结果
     */
    public SysUser checkEmailUnique(String email);

    List<Map<String, Object>> selectOption(SysUser user);

    /***
     * 查询用户信息列表
     * @param sysUser
     * @return
     */
    public List<SysUser> selectUserListByName(SysUser sysUser);

    /***
     * 查询用户信息列表
     * @param sysUser
     * @return
     */
    public List<SysUser> selectUserListByNotWhite(SysUser sysUser);

    /**
     * 根据条件分页查询用户列表
     *
     * @param sysUser 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> noJoinPositionByList(SysUser sysUser);


    /**
     * 解除用户绑定职位
     *
     * @param user 用户信息
     * @return 结果
     */
    public int relieveUserByPosition(SysUser user);

    /**
     *  获取安全负责人用户列表
     *
     * @param sysUser 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> getSafeUserList(SysUser sysUser);
}
